#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from py_chatgpt_plus.core.image_gpt import ImageGpt

api_key = "****"


def test_image_generate_url():
    ig = ImageGpt(
        api_key=api_key
    )
    urls = ig.generate_url('Draw a white cat. it is a real cat, not a cartoon cat')
    print(urls)


def test_image_generate_content():
    ig = ImageGpt(
        api_key=api_key
    )
    contents = ig.generate_content('Draw a white cat. it is a real cat, not a cartoon cat')
    for i, content in enumerate(contents):
        with open(f"image_{i}.png", "wb") as f:
            f.write(content)


def test_image_generate_variations_url(image_path: str):
    ig = ImageGpt(
        api_key=api_key
    )
    urls = ig.generate_variations_url(image_path)
    print(urls)


if __name__ == "__main__":
    # test_image_generate_url()
    # test_image_generate_content()
    test_image_generate_variations_url("image_0.png")



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
import asyncio
import websockets

from loguru import logger

from py_chatgpt_plus.utils.conf import conf_inst
from py_chatgpt_plus.core.chat_gpt_3 import ChatGptV3

cg = ChatGptV3(
    api_key=conf_inst.chatgpt_api_key,
    proxy_address=conf_inst.chatgpt_proxy)


async def chat_stream(websocket):
    async for message in websocket:
        logger.debug(f"chat_stream | receive prompt:{message}")
        streams = cg.chat_stream(prompt=message)
        all_stream_content = ""
        for stream in streams:
            all_stream_content += stream
            await websocket.send(stream)
        logger.debug(f"chat_stream | send rsp:{all_stream_content}")


async def chat_once(websocket):
    async for message in websocket:
        logger.debug(f"chat_once | receive prompt:{message}")
        rsp = cg.chat_once(prompt=message)
        logger.debug(f"chat_once | send rsp:{rsp}")
        await websocket.send(rsp)


async def handle(websocket, path):
    if path == "/chat_once":
        await chat_once(websocket)
    elif path == "/chat_stream":
        await chat_stream(websocket)
    else:
        raise ValueError(f"not support path:{path}")


async def main():
    logger.debug(f"service will listen on:{conf_inst.local_ip}:{conf_inst.local_port}")
    async with websockets.serve(handle, conf_inst.local_ip, conf_inst.local_port):
        await asyncio.Future()

if __name__ == '__main__':
    asyncio.run(main())



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from typing import Any
import os

import yaml

from py_chatgpt_plus.utils.base_utils import Singleton

def read_yaml(path: str) -> Any:
    with open(path, "rb") as f:
        cf = f.read()
    cf = yaml.load(cf, Loader=yaml.SafeLoader)
    return cf

@Singleton
class ConfParse(object):
    def __init__(self):
        self._local_ip: str = "0.0.0.1"
        self._local_port: int = 32675
        self._chatgpt_api_key: str = None
        self._chatgpt_proxy: str = None
        self._log_file: str = None

    def __call__(self, path: str):
        cf = read_yaml(path)
        server_conf = cf.get("server")
        chatgpt_conf = cf.get("chatgpt")
        log_conf = cf.get("log")
        self._local_ip = server_conf.get("ip", "0.0.0.0")
        self._local_port = server_conf.get("port", 32675)
        self._chatgpt_api_key = chatgpt_conf.get("api_key", "")
        self._chatgpt_proxy = chatgpt_conf.get("proxy", "")
        self._log_file = log_conf.get("log_file", "service.log")
        self._log_level = log_conf.get("level", "DEBUG")

    @property
    def local_ip(self) -> str:
        return self._local_ip

    @property
    def local_port(self) -> int:
        return self._local_port

    @property
    def chatgpt_api_key(self) -> str:
        return self._chatgpt_api_key

    @property
    def chatgpt_proxy(self) -> str:
        return self._chatgpt_proxy

    @property
    def log_file(self) -> str:
        return self._log_file

    @property
    def log_level(self) -> str:
        return self._log_level


conf_inst = ConfParse()
conf_inst(
    path=os.getenv("SERVICE_C0NF", default="../conf/service.yaml")
)


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from loguru import logger
from pathlib import Path
import importlib
import re

from flask import Blueprint

from py_chatgpt_plus.routes.api import app, api_blue
from py_chatgpt_plus.utils.conf import conf_inst


def auto_register_blue_print(app):
    for path in Path(__file__).parent.glob("*_route.py"):
        page_name = re.sub("$%s" % "_route", "", path.stem)
        module_name = ".".join(
            path.parts[path.parts.index("py_chatgpt_plus"): -1] + (page_name,)
        )
        auto_blueprint = importlib.import_module(module_name)
        print(module_name)
        print(page_name)
        auto_blueprint.blue_route = Blueprint(page_name, module_name)
        app.register_blueprint(auto_blueprint.blue_route, url_prefix=f"/{page_name}")


auto_register_blue_print(app)
app.register_blueprint(api_blue)


def main():
    logger.add(conf_inst.log_file, level=conf_inst.log_level, enqueue=True, serialize=False, rotation="100 MB")
    logger.info(f"service will listen on:{conf_inst.local_ip}:{conf_inst.local_port}")
    app.run(host=conf_inst.local_ip, port=conf_inst.local_port)


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from typing import List
import base64

from py_chatgpt_plus.utils.http_call import HttpCall


class ImageGpt(object):
    _image_gpt_url = "https://api.openai.com/v1/images/generations"
    _image_variations_url = "https://api.openai.com/v1/images/variations"

    def __init__(self,
                 api_key: str,
                 proxy_address: str = None,
                 ):
        """
        Creates an image given a prompt
        Args:
            api_key(str): The chatGpt api key, please refer https://platform.openai.com/account/api-keys.
            proxy_address(str): The address of proxy.
        """
        self._api_key = api_key
        self._proxy_address = proxy_address
        self._http_call = HttpCall(proxy_addr=proxy_address)

    def generate_url(self, prompt: str,
                     n: int = 1,
                     size: str = "1024x1024") -> List[str]:
        """
        Generate Image url.
        Args:
            prompt(str): Type prompt.
            n(int): The number of images to generate. Must be between 1 and 10.
            size(str): The size of the generated images. Must be one of 256x256, 512x512, or 1024x1024.
        Returns:
            List[str]: The list of generate image url.
        """
        content = self._http_call.post_once(
            url=self._image_gpt_url,
            header={
                "Authorization": f"Bearer {self._api_key}"
            },
            json={
                "prompt": prompt,
                "n": n,
                "size": size,
                "response_format": "url",
            }
        )
        urls = []
        for d in content["data"]:
            urls.append(d["url"])
        return urls

    def generate_content(self, prompt: str,
                         n: int = 1,
                         size: str = "1024x1024") -> List[bytes]:
        """
        Generate Image content, can be save to a file.
        Args:
            prompt(str): Type prompt.
            n(int): The number of images to generate. Must be between 1 and 10.
            size(str): The size of the generated images. Must be one of 256x256, 512x512, or 1024x1024.
        Returns:
            List[str]: The list of generate image content.
        """
        content = self._http_call.post_once(
            url=self._image_gpt_url,
            header={
                "Authorization": f"Bearer {self._api_key}"
            },
            json={
                "prompt": prompt,
                "n": n,
                "size": size,
                "response_format": "b64_json",
            }
        )
        b64_list = []
        for d in content["data"]:
            b64_list.append(base64.b64decode(d["b64_json"]))
        return b64_list

    def generate_variations_url(self,
                                image_path: str,
                                ) -> List[bytes]:
        """
        Generate a variation image url of a given image.
        Args:
            image_path(str): The image to use as the basis for the variation(s).
             Must be a valid PNG file, less than 4MB, and square.
        Returns:
            List[str]: The list of generate image content.
        """
        content = self._http_call.post_once(
            url=self._image_variations_url,
            header={
                "Authorization": f"Bearer {self._api_key}"
            },
            files={
                    "image": ("gpt_image.png", open(image_path, 'rb'), "image/png"),
                  }
        )
        urls = []
        for d in content["data"]:
            urls.append(base64.b64decode(d["url"]))
        return urls


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
import requests
from typing import Any, Optional, Generator


class HttpCall(object):
    def __init__(self, proxy_addr: str):
        self._session = requests.Session()
        if proxy_addr is not None or not proxy_addr:
            self._session.proxies.update({
                "http": proxy_addr,
                "https": proxy_addr,
            })

    def post_once(self, url: str,
                  header: dict = {"Content-Type": "application/json"},
                  json: Optional[Any] = None,
                  data=None,
                  files=None):
        """
        Non-streaming post interface.
        """
        if json is not None:
            header["Content-Type"] = "application/json"
        rsp = self._session.post(
            url=url,
            headers=header,
            data=data,
            json=json,
            files=files,
            stream=False,
        )
        rsp.raise_for_status()
        return rsp.json()

    def post_stream(self, url: str,
                    header: dict = {"Content-Type": "application/json"},
                    json: Optional[Any] = None,
                    data=None,
                    files=None) -> Generator:
        """
        Streaming post interface
        """
        if json is not None:
            header["Content-Type"] = "application/json"
        rsp = self._session.post(
            url=url,
            headers=header,
            data=data,
            files=files,
            json=json,
            stream=True,
        )
        rsp.raise_for_status()
        for line in rsp.iter_lines():
            if not line:
                continue
            yield line



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
import uuid
from typing import Any

from flask import Flask, Blueprint
from flask_restx import Api, fields

from py_chatgpt_plus.errors.error import ChatGptError, Success

app = Flask(__name__)

app.config.SWAGGER_UI_OPERATION_ID = True  # type: ignore
app.config.SWAGGER_UI_REQUEST_DURATION = True  # type: ignore
app.url_map.strict_slashes = False
app.config["JSON_AS_ASCII"] = False
app.config["ERROR_INCLUDE_MESSAGE"] = False  # 必须设置为False
app.config["SECRET_KEY"] = "@&^&N908jksd#"

api_blue = Blueprint("api", __name__, url_prefix="/api/v1")

api = Api(
    api_blue,
    version="1.0",
    title="ChatGpt service!",
    description="ChatGpt service!",
    doc="/doc",
)


@api.errorhandler(ChatGptError)
def chat_gpt_error_response(e):
    return get_json_result(e), 200


@api.errorhandler(Exception)
def server_internal_err_response(e):
    return "internal server error", 500


def get_json_result(
    custom_err: ChatGptError = Success.SuccessResponse, data=None
):
    """Get API json result.

    Args:
        custom_err (ChatGptError): The Error class include ret_msg and ret_code.
        data (Any): The custom result data.

    Returns:
    """
    result_dict = {"ret_code": custom_err.ret_code(), "ret_msg": custom_err.ret_msg()}
    if data is not None:
        result_dict["data"] = data
    return result_dict


def custom_response(data: Any = None) -> Any:
    if data is not None:
        custom_model = api.model(
            "CustomResponse" + str(uuid.uuid4()),
            {
                "ret_code": fields.Integer(description="The code of result", default=0),
                "ret_msg": fields.String(
                    description="The message of result", default="success"
                ),
                "data": fields.Nested(data),
            },
        )
    else:
        custom_model = api.model(
            "CustomResponse",
            {
                "ret_code": fields.Integer(description="The code of result", default=0),
                "ret_msg": fields.String(
                    description="The message of result", default="success"
                ),
            },
        )
    return custom_model



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from flask import Response, request
from flask_restx import Resource, fields

from loguru import logger

from py_chatgpt_plus.services.chat import ChatService
from py_chatgpt_plus.routes.api import api, custom_response, get_json_result
from py_chatgpt_plus.utils.conf import conf_inst

ns = api.namespace("chat", description="Chat  API")

chat_service = ChatService(api_key=conf_inst.chatgpt_api_key)

chat_base_request_fields = api.clone(
    "ChatBaseRequest",
    {
        "prompt": fields.String(required=True, description="Type prompt."),
        "system_prompt": fields.String(required=True,
                                       description="System prompt",
                                       default="You are ChatGPT, a large language model trained by OpenAI. Respond conversationally"),
    }
)

chat_base_once_response_fields = api.model(
     "ChatBaseOnceResponseFields",
     {
        "answer": fields.String(required=True, description="The id of mpc job."),
     }
)


@ns.route("/once")
class ChatOnce(Resource):
    @ns.expect(chat_base_request_fields)
    @ns.marshal_with(custom_response(chat_base_once_response_fields))
    def post(self):
        """chat once, no stream!"""
        req_json = request.json
        logger.debug(f"receive chat once post request:{req_json}")
        answer = chat_service.chat_once(prompt=req_json.get("prompt", ""),
                                        system_prompt=req_json.get("system_prompt"))
        return get_json_result(data={"answer": answer})


@ns.route("/stream")
class ChatStream(Resource):
    @api.doc(responses={200: "Stream SSE data in text/event-stream format"})
    @api.representation("text/event-stream")
    @ns.expect(chat_base_request_fields)
    def post(self):
        """chat stream."""
        req_json = request.json
        logger.debug(f"receive chat stream post request:{req_json}")
        return Response(chat_service.chat_stream(prompt=req_json.get("prompt", ""),
                                                 system_prompt=req_json.get("system_prompt")),
                                                 mimetype="text/event-stream")


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
import argparse
import importlib
import inspect
import sys
from typing import Dict, List, Union

from py_chatgpt_plus.errors import ChatGptError
from pytablewriter import MarkdownTableWriter

module_paths = ["py_chatgpt_plus.errors.error"]


def dynamic_import(module):
    return importlib.import_module(module)


def get_module_attributes(module_path: str):
    errcode_dict: Dict[str, List[Dict[str, Union[str, int]]]] = {}
    module = dynamic_import(module_path)
    for class_name, class_ in inspect.getmembers(
        sys.modules[module.__name__], inspect.isclass
    ):
        attributes = inspect.getmembers(class_, lambda a: not (inspect.isroutine(a)))
        attr_list = [
            a for a in attributes if not (a[0].startswith("__") and a[0].endswith("__"))
        ]
        for attr in attr_list:
            if isinstance(attr[1], ChatGptError):
                module_name = attr[0]
                ret_code = attr[1].__dict__["_ret_code"]
                ret_msg = attr[1].__dict__["_ret_msg"]
                if class_name not in errcode_dict:
                    errcode_dict[class_name] = [
                        {
                            "ret_code": ret_code,
                            "ret_msg": ret_msg,
                            "ret_module": module_name,
                        }
                    ]
                else:
                    errcode_dict[class_name].append(
                        {
                            "ret_code": ret_code,
                            "ret_msg": ret_msg,
                            "ret_module": module_name,
                        }
                    )
    return errcode_dict


def export(dst_file: str):
    errcode_api_md_content = "# Server Errcode API    \n"
    for module_path in module_paths:
        errcode_dict = get_module_attributes(module_path)
        for className, code_info_lists in errcode_dict.items():
            writer = MarkdownTableWriter(
                table_name=className,
                headers=["ret_module", "ret_code", "ret_msg"],
                value_matrix=[],
                margin=1,  # add a whitespace for both sides of each cell
            )
            value_matrix = []
            for code_info_dict in code_info_lists:
                value_matrix.append(
                    [
                        code_info_dict["ret_module"],
                        code_info_dict["ret_code"],
                        code_info_dict["ret_msg"],
                    ]
                )
            writer.value_matrix = value_matrix
            content = writer.dumps()
            errcode_api_md_content += str(content) + "    \n"
    with open(dst_file, "w") as md_file:
        md_file.write(errcode_api_md_content)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Export errcode")
    parser.add_argument("-f", "--dst_file", type=str, required=True, help="output file")
    args = parser.parse_args()

    export(dst_file=args.dst_file)



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
class ChatGptError(Exception):
    def __init__(self, ret_code: int = 0, ret_msg: str = "success"):
        """Define base error.

        Args:
            ret_code (int): The code of ret, default to 0.
            ret_msg: The description of ret, default to success.
        """
        self._ret_code = ret_code
        self._ret_msg = ret_msg

    def ret_code(self) -> int:
        return self._ret_code

    def ret_msg(self) -> str:
        return self._ret_msg

    def set_ret_msg(self, msg: str):
        self._ret_msg = msg
        return self


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from typing import Generator
from py_chatgpt_plus.core.chat_gpt_3 import ChatGptV3


class ChatService(object):
    def __init__(self, api_key: str):
        self._api_key = api_key

    def chat_once(self, prompt: str, system_prompt: str) -> str:
        cg = ChatGptV3(api_key=self._api_key,
                       system_prompt=system_prompt)
        return cg.chat_once(prompt=prompt)

    def chat_stream(self, prompt: str, system_prompt: str) -> Generator:
        cg = ChatGptV3(api_key=self._api_key)
        return cg.chat_stream(prompt=prompt,
                              system_prompt=system_prompt)


#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
import json
from typing import List, Generator

from py_chatgpt_plus.core.chat_base import ChatAbc
from py_chatgpt_plus.utils.http_call import HttpCall


class ChatGptV3(ChatAbc):
    def __init__(self,
                 api_key: str,
                 model: str = "gpt-3.5-turbo",
                 chatgpt_url: str = "https://api.openai.com/v1/chat/completions",
                 system_prompt: str = "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally",
                 proxy_address: str = None,
                 temperature: float = 0.6,
                 top_p: float = 1.0,
                 max_tokens: int = 2048,
                 ):
        """
        Call Official chatGpt interface.
        Args:
            api_key(str): The chatGpt api key, please refer https://platform.openai.com/account/api-keys.
            model(str): The model of chatGpt v3, default to 'gpt-3.5-turbo'.
            proxy_address(str): The address of proxy.
            system_prompt(str): You are a
            temperature(float): What sampling temperature to use, between 0 and 2.\
                  Higher values like 0.8 will make the output more random, while lower values like 0.2 will \
                  make it more focused and deterministic.
            top_p(float): An alternative to sampling with temperature, called nucleus sampling, where the model
                 considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens \
                 comprising the top 10% probability mass are considered.
            max_tokens(int): The maximum number of tokens to generate in the completion.The token count of your \
                 prompt plus max_tokens cannot exceed the model's context length. Most models have a context length \
                 of 2048 tokens (except for the newest models, which support 4096).
        """
        self._api_key = api_key
        self._model = model
        self._proxy_address = proxy_address
        self._temperature = temperature
        self._top_p = top_p
        self._max_tokens = max_tokens
        self._http_call = HttpCall(proxy_addr=proxy_address)
        self._system_prompt = system_prompt
        self._conversations: List[dict] = [
            {
                "role": "system",
                "content": self._system_prompt,
            }
        ]
        self._chatgpt_url = chatgpt_url

    def _add_to_conversations(self, content: str, role: str = "user"):
        self._conversations.append(
            {
                "role": role,
                "content": content,
            }
        )

    def chat_once(self, prompt: str) -> str:
        all_rsp: str = ""
        for stream in self.chat_stream(prompt):
            all_rsp += stream
        return all_rsp

    def chat_stream(self, prompt: str) -> Generator:
        self._add_to_conversations(prompt, "user")
        streams = self._http_call.post_stream(
            url=self._chatgpt_url,
            header={
                "Authorization": f"Bearer {self._api_key}"
            },
            json={
                "model": self._model,
                "messages": self._conversations,
                "stream": True,
                "temperature": self._temperature,
                "top_p": self._top_p,
                "user": "user",
                "max_tokens": self._max_tokens,
            }
        )
        all_stream_content = ""
        rsp_role: str = None
        for stream in streams:
            stream = stream.decode("utf-8")[6:]
            if stream == "[DONE]":
                break
            rsp: dict = json.loads(stream)
            choices = rsp.get("choices")
            if not choices:
                continue
            delta = choices[0].get("delta")
            if not delta:
                continue
            if "role" in delta:
                rsp_role = delta["role"]
            if "content" in delta:
                content = delta["content"]
                all_stream_content += content
                yield content
        self._add_to_conversations(all_stream_content, rsp_role)

    def save_conversations(self, file_path: str):
        conversations_json = json.dumps(self._conversations, indent=4, ensure_ascii=False)
        with open(file_path, "w") as f:
            f.write(conversations_json)

    def load_conversations(self, file_path: str):
        with open(file_path, "r") as f:
            content = f.read()
        self._conversations = json.loads(content)


from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

import setuptools
from setuptools import find_packages
import os

def _process_requirements():
    packages = open('requirements.txt').read().strip().split('\n')
    requires = []
    for pkg in packages:
        if pkg.startswith('git+ssh'):
            return_code = os.system('pip install {}'.format(pkg))
            assert return_code == 0, 'error, status_code is: {}, exit!'.format(return_code)
        else:
            requires.append(pkg)
    return requires

with open("../README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py-chatgpt-plus",
    version="0.2.0",
    author="rosemary666",
    author_email="",
    description="Python chatgpt api",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rosemary666/chatgpt",
    packages=[
        package for package in find_packages()
        if package.startswith('py_chatgpt_plus')
    ],
    install_requires=_process_requirements(),
    setup_requires=[],
    license="Apache License 2.0",
    classifiers=[
        'Development Status :: 3 - Alpha',
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    python_requires='>=3.7',
)



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from abc import ABC, abstractmethod
from typing import Generator


class ChatAbc(ABC):
    """
    The chat abstract class encapsulated by calling the chatGpt interface.
    """
    @abstractmethod
    def chat_once(self, prompt: str) -> str:
        """
        Return results all at once, non-streaming return.
        Args:
            prompt(str): Typed prompt.

        Returns:
            str: The reply.
        """
        pass

    @abstractmethod
    def chat_stream(self, prompt: str) -> Generator:
        """
        Return result by stream.
        Args:
            prompt(str): Typed prompt.

        Returns:
            The reply stream.
        """
        pass

    @abstractmethod
    def save_conversations(self, file_path: str):
        """
        Save the conversations to a specified file.
        Args:
            file_path(str): The path of file.

        Returns:
        """
        pass

    @abstractmethod
    def load_conversations(self, file_path: str):
        """
        Load the conversations from a specified file.
        Returns:

        """
        pass



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
"""Export swagger schema."""
import argparse
import json

from py_chatgpt_plus.routes.api import api, app

def export_schema_to_swagger(dst_file: str):
    """Export schema to swagger json file.

    Args:
        dst_file: The output file.

    Returns:
    """
    app.config["SERVER_NAME"] = "localhost"
    app.app_context().__enter__()
    with open(dst_file, "w") as json_file:
        json.dump(api.__schema__, json_file, indent=4)


def export_schema_to_postman_collection(dst_file: str):
    """Export API schema as a Postman collection.

    Args:
        dst_file: The output file.

    Returns:
    """
    app.config["SERVER_NAME"] = "localhost"
    app.app_context().__enter__()
    urlvars = False  # Build query strings in URLs
    swagger = True  # Export Swagger specifications
    data = api.as_postman(urlvars=urlvars, swagger=swagger)
    with open(dst_file, "w") as json_file:
        json.dump(data, json_file, indent=2)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Export swagger schema")
    parser.add_argument(
        "-t",
        "--type",
        type=int,
        default=0,
        choices=[0, 1],
        help="export as swagger or postman collection, 0 for swagger, 1 for postman collection",
    )
    parser.add_argument("-f", "--dst_file", type=str, required=True, help="output file")
    args = parser.parse_args()

    if args.type == 0:
        export_schema_to_swagger(args.dst_file)
    elif args.type == 1:
        export_schema_to_postman_collection(args.dst_file)
    else:
        raise Exception("unsupported export type.")



#  Copyright 2023 github.com/rosemary666. All Rights Reserved.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#      http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ==============================================================================
from py_chatgpt_plus.core.chat_gpt_3 import ChatGptV3

api_key = "*******"


def test_stream():
    cg = ChatGptV3(
        api_key=api_key
    )
    streams = cg.chat_stream("请详细介绍一下你自己")
    for stream in streams:
        print(stream)


def test_once():
    cg = ChatGptV3(
        api_key=api_key,
        system_prompt="请帮我把以下的工作内容填充为工作周报，用markdown格式以分点叙述的方式输出:",
    )
    answer = cg.chat_once("主要开展了三件事情, 第一是完成功能1的开发，第二是完成了xx的需求评审，第三是接待了xx客户")
    print(answer)
    cg.save_conversations("./conversations.json")


if __name__ == "__main__":
    # test_stream()
    test_once()